from PySide6.QtWidgets import (
  QApplication, QMainWindow, QWidget,
  QVBoxLayout, QHBoxLayout, QPushButton,
  QTextEdit, QLineEdit,
  QLabel, QFileDialog,
)
from PySide6.QtCore import Qt, QThread, Signal
import os, sys, time, re
import google.generativeai as genai
from dotenv import load_dotenv
from markdown2 import markdown
from PySide6.QtGui import QFont

BACKGROUND_COLOR = "white"
BORDER_COLOR = "#B2D8B5"
BUTTON_COLOR = "#06C755"
BUTTON_HOVER_COLOR = "#66BB6A"
DEFAULT_FONT_SIZE = 10
MAX_HISTORY_LENGTH = 10

class ChatWindow(QMainWindow):
  def __init__(self):
    super().__init__()
    self.history = []
    self.setWindowTitle("ChatBot")
    self.setGeometry(100, 100, 600, 400)
    self.central_widget = QWidget()
    self.setCentralWidget(self.central_widget)
    self.main_layout = QVBoxLayout()
    self.chat_layout = QVBoxLayout()
    self.input_layout = QHBoxLayout()
    self.file_layout = QHBoxLayout()
    self.chat_display = QTextEdit()
    self.chat_display.setReadOnly(True)
    self.main_layout.addWidget(self.chat_display)
    self.input_box = QLineEdit()
    self.input_box.setPlaceholderText("ここにメッセージを入力")
    self.input_layout.addWidget(self.input_box)
    self.send_button = QPushButton("送信")
    self.send_button.clicked.connect(self.handle_send)
    self.input_layout.addWidget(self.send_button)
    self.main_layout.addLayout(self.input_layout)
    self.file_label = QLabel("ファイルが選択されていません")
    self.file_label.setAlignment(Qt.AlignLeft)
    self.file_layout.addWidget(self.file_label)
    self.upload_button = QPushButton("ファイルを選択")
    self.upload_button.clicked.connect(self.handle_file_selection)
    self.file_layout.addWidget(self.upload_button)
    self.main_layout.addLayout(self.file_layout)
    self.selected_file_path = None
    self.central_widget.setLayout(self.main_layout)
    self.file_label.setStyleSheet("color: #4A4A4A;")
    self.chat_display.setStyleSheet(
      f"background-color: {BACKGROUND_COLOR}; "
      f"border: 1px solid {BORDER_COLOR}; "
      "border-radius: 5px; padding: 10px;"
    )
    self.input_box.setStyleSheet(
      f"background-color: {BACKGROUND_COLOR}; "
      f"border: 1px solid {BORDER_COLOR}; "
      "border-radius: 5px; padding: 5px;"
    )
    self.send_button.setStyleSheet(
      "QPushButton {"
      f"    background-color: {BUTTON_COLOR};"
      "    color: white; border: none;"
      "    border-radius: 5px; padding: 5px 10px;"
      "}"
      "QPushButton:hover {"
      f"    background-color: {BUTTON_HOVER_COLOR};"
      "}"
    )
    self.upload_button.setStyleSheet(
      "QPushButton {"
      f"    background-color: {BUTTON_COLOR};"
      "    color: white; border: none;"
      "    border-radius: 5px; padding: 5px 10px;"
      "}"
      "QPushButton:hover {"
      f"    background-color: {BUTTON_COLOR};"
      "}"
    )
    self.update_font_sizes()
  def resizeEvent(self, event):
    super().resizeEvent(event)
    self.update_font_sizes()
  def update_font_sizes(self):
    height = self.height()
    new_font_size = max(DEFAULT_FONT_SIZE, height // 50)
    font = QFont()
    font.setPointSize(new_font_size)
    self.chat_display.setFont(font)
    self.input_box.setFont(font)
    self.file_label.setFont(font)
    self.upload_button.setFont(font)
    self.send_button.setFont(font)
  def handle_send(self):
    user_message = self.input_box.text().strip()
    if not user_message: return
    self.send_button.setEnabled(False)
    self.chat_display.append(f"<b>あなた:</b> {user_message}")
    if self.selected_file_path:
      self.chat_display.append(
        f"<b>添付ファイル:</b> {self.selected_file_path}"
      )
      file_path = self.selected_file_path
      self.selected_file_path = None
      self.file_label.setText("ファイルが選択されていません")
    else:
      file_path = None
    self.input_box.clear()
    self.response_thread = BotResponseThread(
      user_message, self.history, file_path
    )
    self.response_thread.response_ready.connect(
      self.display_bot_response
    )
    self.response_thread.start()
  def safe_markdown(self, text):
    lines = text.splitlines()
    formatted_lines = []
    for i, line in enumerate(lines):
      if re.match(r"^\s*(\d+\.\s+|[-*+]\s+)", line) and (
        i + 1 < len(lines) and lines[i + 1].strip()
      ):
        formatted_lines.append(line)
        formatted_lines.append("")
      else:
        formatted_lines.append(line)
    formatted_text = "\n".join(formatted_lines)
    return markdown(formatted_text)
  def display_bot_response(self, response_text):
    html_content = self.safe_markdown(response_text)
    self.chat_display.append(f"<b>Bot:</b> {html_content}")
    self.send_button.setEnabled(True)
  def handle_file_selection(self):
    file_dialog = QFileDialog()
    file_path, _ = file_dialog.getOpenFileName(
      self, "ファイルを選択"
    )
    if file_path:
      self.selected_file_path = file_path
      self.file_label.setText(
        f"選択したファイル: {file_path.split('/')[-1]}"
      )

class BotResponseThread(QThread):
  response_ready = Signal(str)
  def __init__(self, user_message, history, file_path=None):
    super().__init__()
    self.user_message = user_message
    self.file_path = file_path
    self.history = history
  def run(self):
    response_text = self.get_bot_response(
      self.user_message, self.file_path
    )
    self.response_ready.emit(response_text)
  def get_bot_response(self, message, file_path=None):
    input_text = message
    self.history.append({"role": "user", "parts": [input_text]})
    if file_path:
      attached_file = genai.upload_file(path=file_path)
      while attached_file.state.name == "PROCESSING":
        time.sleep(10)
        attached_file = genai.get_file(attached_file.name)
      self.history.append(
        {"role": "user", "parts": [attached_file]}
      )
    else:
      pass
    response = model.generate_content(self.history)
    self.history.append(response.candidates[0].content)
    if len(self.history) > MAX_HISTORY_LENGTH:
      self.history.pop(0)
    return response.text

def initialize_genai():
  load_dotenv()
  api_key = os.getenv("GOOGLE_API_KEY")
  genai.configure(api_key=api_key)
  return genai.GenerativeModel("gemini-1.5-flash")

if __name__ == "__main__":
  model = initialize_genai()
  app = QApplication(sys.argv)
  window = ChatWindow()
  window.show()
  sys.exit(app.exec())
